# Example: Definition of a PCell with Properties
import si_fab.all as pdk  # noqa
from ipkiss3 import all as i3


# Get the technology
# We will define a Ring Resonator PCell which consists of
# a ring waveguide with evanescent coupling to a bus waveguide.
class RingResonator(i3.PCell):
    """A generic ring resonator class.

    It is defined by a circular waveguide evanescently coupled to a straight bus waveguide.
    """

    _name_prefix = "RINGRES"  # a prefix added to the unique identifier

    # We moved the properties from PCell level to LayoutView level as
    # ring_radius, ring_wg_width, bus_wg_width and coupler_spacing
    # are only relevant in the context of Layout.

    class Layout(i3.LayoutView):
        # In this example we use the technology
        ring_radius = i3.PositiveNumberProperty(default=i3.TECH.WG.BEND_RADIUS, doc="radius of the ring in micrometers")
        ring_wg_width = i3.PositiveNumberProperty(
            default=i3.TECH.WG.WIRE_WIDTH, doc="width of the ring waveguide in micrometers"
        )
        bus_wg_width = i3.PositiveNumberProperty(
            default=i3.TECH.WG.WIRE_WIDTH, doc="width of the bus waveguide in micrometers"
        )
        coupler_spacing = i3.PositiveNumberProperty(
            default=i3.TECH.WG.DC_SPACING, doc="spacing between centerline of bus waveguide and ring waveguide"
        )

        def validate_properties(self):
            """Check whether the combination of properties is valid."""
            if self.coupler_spacing <= 0.5 * (self.ring_wg_width + self.bus_wg_width):
                raise i3.PropertyValidationError(
                    self,
                    "coupler_spacing too small: waveguides will touch",
                    {
                        "coupler_spacing": self.coupler_spacing,
                        "ring_wg_width": self.ring_wg_width,
                        "bus.wg_width": self.bus_wg_width,
                    },
                )

            if self.ring_radius < self.ring_wg_width:
                raise i3.PropertyValidationError(
                    self,
                    "ring_radius too small for given waveguide definition",
                    {"ring_radius": self.ring_radius, "ring_wg_width": self.ring_wg_width},
                )
            return True

        # This method generates the layout by adding elements to "elems"
        def _generate_elements(self, elems):
            # Here, we add the layout elements based shapes: two paths on
            # the same layer, with the right width.

            r = self.ring_radius
            s = self.coupler_spacing

            # 1. We build the ring shape and a bus shape add it to the layout elements.
            shape_ring = i3.ShapeCircle(center=(0, 0), radius=r)
            shape_bus = [(-r, -r - s), (+r, -r - s)]

            # 2. We add the shapes to elems.
            elems += i3.Path(layer=i3.TECH.PPLAYER.SI, shape=shape_ring, line_width=self.ring_wg_width)
            elems += i3.Path(layer=i3.TECH.PPLAYER.SI, shape=shape_bus, line_width=self.bus_wg_width)

            # 3. We return elems
            return elems


if __name__ == "__main__":
    print("This is not the main file. Run 'execute.py' in the same folder")
